from django import forms
from django.contrib.auth.models import User
from django.contrib.auth.forms import UserCreationForm, UserChangeForm, PasswordResetForm
from django.utils.translation import gettext_lazy as _
from .signals import send_activation_email
from threading import Thread
from .defaults import getattribute


class UserRegisterForm(UserCreationForm):
    email = forms.EmailField()
    class Meta:
        model = User
        fields = ['username', 'email', 'password1', 'password2']

    if getattribute('UNIQUE_EMAIL'):
        def clean(self):
            super(UserRegisterForm, self).clean()
            email = self.cleaned_data.get('email')
            users = User.objects.filter(email__iexact=email)
            if users.exists():
                user = users.last()
                msg = _("Email already exists!")
                if user.is_active == False:
                    thread = Thread(target=send_activation_email, args=(user,))
                    thread.setDaemon(True)
                    thread.start()
                    msg = _("Email already exists, Activation link has been sent to email!")
                self.add_error('email', msg)
            return self.cleaned_data


class UserProfile(UserChangeForm):
    password = None
    class Meta:
        model = User
        fields = ('first_name', 'last_name', 'username', 'email')
        widgets = {
            'first_name': forms.TextInput(attrs={'class': 'form-control mt-1', 'placeholder': 'First Name'}),
            'last_name': forms.TextInput(attrs={'class': 'form-control mt-1', 'placeholder': 'Last Name'}),
            'username': forms.TextInput(attrs={'class': 'form-control mt-1', 'placeholder': 'Username'}),
            'email': forms.EmailInput(attrs={'class': 'form-control mt-1', 'placeholder': 'Email'}),
        }


class EmailValidationCheckForm(PasswordResetForm):
    def clean_email(self):
        email = self.cleaned_data['email']
        if not User.objects.filter(email__iexact=email).exists():
            msg = _("There is no user registered with the specified E-Mail address.")
            self.add_error('email', msg)
        return email



from django.contrib.auth.backends import ModelBackend
from django.db.models import Q
from django.contrib.auth.models import User


class EmailOrUsernameModelBackend(ModelBackend):
    """
    Authentication backend which allows users to authenticate using either their
    email or username
    """

    def authenticate(self, request, username=None, password=None):
        try:
            user = User.objects.get(
                Q(email=username) | Q(username=username)
            )
            return user if user.check_password(password) else None
        except User.DoesNotExist:
            return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.urls import path
from django.contrib.auth import views as auth_views
from .defaults import getattribute
from .views import (
    SignUp, Login, ActivateUser, Profile,
    PasswordChange, PasswordReset, PasswordResetConfirm
    )

signupattr =  getattribute('SIGNUP_URL')
signup = 'signup/' if signupattr == True else signupattr
signinattr = getattribute('SIGNIN_URL')
signin = 'signin/' if signinattr == True else signinattr

urlpatterns = [
    path(signup, SignUp.as_view(), name='signup'),
    path(signin, Login.as_view(), name='signin'),
    path('logout/', auth_views.LogoutView.as_view(), name='logout'),
    path('profile/', Profile.as_view(), name='profile'),
    path('change-password/', PasswordChange.as_view(), name='change-password'),
    path('forget/', PasswordReset.as_view(), name='reset-password'),
    path('forget/<uidb64>/<token>/', PasswordResetConfirm.as_view(), name='password_reset_confirm'),
]
if getattribute('ACCOUNT_ACTIVATION'): urlpatterns.append(
    path('account-activation/<str:username>/<str:token>/', ActivateUser.as_view(), name='account-activation')
)



from django.db.models.signals import post_save
from django.dispatch import receiver
from django.contrib.auth.models import User
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from .tokens import account_activation_token
from django.conf import settings
from threading import Thread
from .defaults import getattribute


def send_activation_email(user):
    email_subject = "Activate Your Account"
    email_temp = getattribute('ACTIVATION_EMAIL_TEMPLATE')
    template_name = 'accounts/activate.html' if email_temp==True else email_temp
    current_site = getattribute('CURRENT_SITE')
    email_body = render_to_string(template_name, {
        'user': user,
        "domain": "http://127.0.0.1:8000" if current_site==True else current_site,
        "username": urlsafe_base64_encode(force_bytes(user.username)),
        'token': account_activation_token.make_token(user),
    })
    email = EmailMessage(subject=email_subject, body=email_body, from_email=settings.EMAIL_FROM_USER,
    to=[user.email])
    email.send()


@receiver(post_save, sender=User)
def update_user(sender, instance, created, **kwargs):
    if created and getattribute('ACCOUNT_ACTIVATION') and not instance.is_superuser:
            instance.is_active = False
            instance.save()
            thread = Thread(target=send_activation_email, args=(instance,))
            thread.setDaemon(True)
            thread.start()



from django.views import View
from django.shortcuts import redirect, HttpResponse
from django.contrib import messages
from django.contrib.auth.views import LoginView
from django.views.generic import CreateView, UpdateView
from django.urls import reverse_lazy
from django.contrib.messages.views import SuccessMessageMixin
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.views import PasswordChangeView, PasswordResetView, PasswordResetConfirmView
from django.utils.http import urlsafe_base64_decode
from .tokens import account_activation_token
from django.utils.encoding import force_str
from .forms import (
    UserRegisterForm, UserProfile,
    EmailValidationCheckForm
    )
from django.contrib.auth.models import User
from .signals import send_activation_email
from threading import Thread
from .defaults import getattribute


class SignUp(SuccessMessageMixin, CreateView):
    form_class = UserRegisterForm
    sig_temp = getattribute('SIGNUP_TEMPLATE')
    template_name = sig_temp if sig_temp!=True else "accounts/signup.html"
    success_url = reverse_lazy('signin')
    success_message = "Your account created successfully!"
    if getattribute('ACCOUNT_ACTIVATION'):
        success_message = "Please check your mail to verify!"


class ActivateUser(View):
    def get(self, request, username, token):
        try:
            encUser = force_str(urlsafe_base64_decode(username))
            user = User.objects.filter(username=encUser)
            if user.exists():
                user = user.last()
            else: raise 'Email not found!'
        except Exception:
            user = None

        if user != None:
            if account_activation_token.check_token(user, token):
                user.is_active = True
                user.save()
                messages.add_message(request, messages.SUCCESS, 'Your account successfully activated!')
            else:
                thread = Thread(target=send_activation_email, args=(user,))
                thread.setDaemon(True)
                thread.start()
                messages.add_message(request, messages.SUCCESS, 'Link is expired! A new link sent to your mail!')
            return redirect('signin')
        return HttpResponse('Invalid login!')


class Login(SuccessMessageMixin, LoginView):
    sig_temp = getattribute('LOGIN_TEMPLATE')
    template_name = sig_temp if sig_temp!=True else 'accounts/login.html'
    success_url = reverse_lazy('index')
    success_message = "Login successful!"


class Profile(SuccessMessageMixin, LoginRequiredMixin, UpdateView):
    form_class = UserProfile
    pro_temp = getattribute('PROFILE_TEMPLATE')
    template_name = pro_temp if pro_temp!=True else 'accounts/profile.html'
    success_url = reverse_lazy('index')
    success_message = "Your profile updated successfully!"
    def get_object(self):
        return self.request.user


class PasswordChange(SuccessMessageMixin, LoginRequiredMixin, PasswordChangeView):
    template_name = 'accounts/change-password.html'
    success_url = reverse_lazy('index')
    success_message = "Your password was changed successfully"


class PasswordReset(SuccessMessageMixin, PasswordResetView):
    for_temp = getattribute('FORGET_TEMPLATE')
    template_name = for_temp if for_temp!=True else 'accounts/reset-password.html'
    success_url = reverse_lazy('signin')
    success_message = "Your password reset request was sent"
    form_class = EmailValidationCheckForm


class PasswordResetConfirm(SuccessMessageMixin, PasswordResetConfirmView):
    template_name = 'accounts/reset-password-confirm.html'
    success_url = reverse_lazy('signin')
    success_message = "Your password reset was done successfully"



from setuptools import find_packages, setup


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="dj-acc",
    version="0.0.1",
    description="An extensible user-accounts application for Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Abdul Rehman",
    author_email="adnan1470369258@gmail.com",
    url="https://github.com/ardevpk/djacc",
    project_urls = {
        "Bug Tracker": "https://github.com/ardevpk/djacc/issues",
    },
    
    package_dir={"": "src"},
    include_package_data=True,
    py_modules=['accounts'],
    packages=find_packages("src"),
    package_data={
        'accounts': [
            'templates/accounts/*.html',
            ]
        },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Utilities",
    ],
    python_requires=">=3.7",
    install_requires=[
        "Django>=3.8",
        'django-crispy-forms>=1.14.0',
        'django-utils-six>=2.0'
        ],
)



from django.conf import settings

UNIQUE_EMAIL = False
ACCOUNT_ACTIVATION = False
LOGIN_TEMPLATE = 'path'
SIGNUP_TEMPLATE = 'path/index.html'
ACTIVATION_EMAIL_TEMPLATE = 'path'
FORGET_TEMPLATE = 'path'
SIGNUP_URL = 'register/'
SIGNIN_URL = 'login/'
PROFILE_TEMPLATE = 'path'


UNIQUE_EMAIL = False # True
ACCOUNT_ACTIVATION = False # True
SIGNUP_URL = 'register/' # 'signup'
SIGNIN_URL = 'login/' # 'signin'

SIGNUP_TEMPLATE = 'your-temp-path/temp-name.html' # 'accounts/register.html'
ACTIVATION_EMAIL_TEMPLATE = 'your-temp-path/temp-name.html' # 'accounts/activate.html'
LOGIN_TEMPLATE = 'your-temp-path/temp-name.html' # 'accounts/login.html'
FORGET_TEMPLATE = 'your-temp-path/temp-name.html' # 'accounts/reset-password.html'
PROFILE_TEMPLATE = 'your-temp-path/temp-name.html' # 'accounts/profile.html'

CURRENT_SITE = 'example.com' # 'localhost:8000'


def getattribute(attr):
    return getattr(settings, attr, True)
